@@ -4,18 +4,52 @@ This module solves for the orbit of the planet given Keplerian parameters.
 import numpy as np
 import astropy.units as u
 import astropy.constants as consts
-import warnings # to be removed after tau_ref_epoch warning is removed. 
 
-try:
+from orbitize import cuda_ext, cext
+
+if cext:
     from . import _kepler
-    cext = True
-except ImportError:
-    print("WARNING: KEPLER: Unable to import C-based Kepler's \
-equation solver. Falling back to the slower NumPy implementation.")
-    cext = False
+
+if cuda_ext:
+    # Configure GPU context for CUDA accelerated compute
+    from orbitize import gpu_context
+    kep_gpu_ctx = gpu_context.gpu_context()
+
+def tau_to_manom(date, sma, mtot, tau, tau_ref_epoch):
+    """
+    Gets the mean anomlay
+    
+    Args:
+        date (float or np.array): MJD
+        sma (float): semi major axis (AU)
+        mtot (float): total mass (M_sun)
+        tau (float): epoch of periastron, in units of the orbital period
+        tau_ref_epoch (float): reference epoch for tau
+        
+    Returns:
+        float or np.array: mean anomaly on that date [0, 2pi)
+    """
+
+    period = np.sqrt(
+        4 * np.pi**2.0 * (sma * u.AU)**3 /
+        (consts.G * (mtot * u.Msun))
+    )
+    period = period.to(u.day).value
+
+    frac_date = (date - tau_ref_epoch)/period
+    frac_date %= 1
+
+    mean_anom = (frac_date - tau) * 2 * np.pi
+    mean_anom %= 2 * np.pi
+
+    return mean_anom
 
 
-def calc_orbit(epochs, sma, ecc, inc, aop, pan, tau, plx, mtot, mass_for_Kamp=None, tau_ref_epoch=58849, tolerance=1e-9, max_iter=100, tau_warning=True):
+def calc_orbit(
+  epochs, sma, ecc, inc, aop, pan, tau, plx, mtot, mass_for_Kamp=None, tau_ref_epoch=58849, tolerance=1e-9, 
+  max_iter=100, use_c=True, use_gpu=False
+):
+
     """
     Returns the separation and radial velocity of the body given array of
     orbital parameters (size n_orbs) at given epochs (array of size n_dates)
@@ -39,9 +73,8 @@ def calc_orbit(epochs, sma, ecc, inc, aop, pan, tau, plx, mtot, mass_for_Kamp=No
         tau_ref_epoch (float, optional): reference date that tau is defined with respect to (i.e., tau=0)
         tolerance (float, optional): absolute tolerance of iterative computation. Defaults to 1e-9.
         max_iter (int, optional): maximum number of iterations before switching. Defaults to 100.
-        tau_warning (bool, optional, depricating): temporary argument to warn users about tau_ref_epoch default value change. 
-            Users that are calling this function themsleves should receive a warning since default is True. 
-            To be removed when tau_ref_epoch change is fully propogated to users. Users can turn it off to stop getting the warning.
+        use_c (bool, optional): Use the C solver if configured. Defaults to True
+        use_gpu (bool, optional): Use the GPU solver if configured. Defaults to False
 
     Return:
         3-tuple:
@@ -56,11 +89,6 @@ def calc_orbit(epochs, sma, ecc, inc, aop, pan, tau, plx, mtot, mass_for_Kamp=No
 
     Written: Jason Wang, Henry Ngo, 2018
     """
-    if tau_warning:
-        warnings.warn("tau_ref_epoch default for kepler.calc_orbit is 58849 now instead of 0 MJD. "
-                      "Please check that this does not break your code. You can turn off this warning by setting "
-                      "tau_warning=False when you call kepler.calc_orbit.")
-
     n_orbs = np.size(sma)  # num sets of input orbital parameters
     n_dates = np.size(epochs)  # number of dates to compute offsets and vz
 
@@ -73,15 +101,11 @@ def calc_orbit(epochs, sma, ecc, inc, aop, pan, tau, plx, mtot, mass_for_Kamp=No
         epochs = np.array([epochs])
     ecc_arr = np.tile(ecc, (n_dates, 1))
 
-    # Compute period (from Kepler's third law) and mean motion
-    period = np.sqrt(4*np.pi**2.0*(sma*u.AU)**3/(consts.G*(mtot*u.Msun)))
-    period = period.to(u.day).value
-    mean_motion = 2*np.pi/(period)  # in rad/day
-
     # # compute mean anomaly (size: n_orbs x n_dates)
-    manom = (mean_motion*(epochs[:, None] - tau_ref_epoch) - 2*np.pi*tau) % (2.0*np.pi)
+    manom = tau_to_manom(epochs[:, None], sma, mtot, tau, tau_ref_epoch)
     # compute eccentric anomalies (size: n_orbs x n_dates)
-    eanom = _calc_ecc_anom(manom, ecc_arr, tolerance=tolerance, max_iter=max_iter)
+    eanom = _calc_ecc_anom(manom, ecc_arr, tolerance=tolerance, max_iter=max_iter, use_c=use_c, use_gpu=use_gpu)
+
     # compute the true anomalies (size: n_orbs x n_dates)
     # Note: matrix multiplication makes the shapes work out here and below
     tanom = 2.*np.arctan(np.sqrt((1.0 + ecc)/(1.0 - ecc))*np.tan(0.5*eanom))
@@ -116,7 +140,7 @@ def calc_orbit(epochs, sma, ecc, inc, aop, pan, tau, plx, mtot, mass_for_Kamp=No
     vz = np.squeeze(vz)[()]
     return raoff, deoff, vz
 
-def _calc_ecc_anom(manom, ecc, tolerance=1e-9, max_iter=100, use_c=False):
+def _calc_ecc_anom(manom, ecc, tolerance=1e-9, max_iter=100, use_c=False, use_gpu=False):
     """
     Computes the eccentric anomaly from the mean anomlay.
     Code from Rob De Rosa's orbit solver (e < 0.95 use Newton, e >= 0.95 use Mikkola)
@@ -126,7 +150,10 @@ def _calc_ecc_anom(manom, ecc, tolerance=1e-9, max_iter=100, use_c=False):
         ecc (float/np.array): eccentricity, either a scalar or np.array of the same shape as manom
         tolerance (float, optional): absolute tolerance of iterative computation. Defaults to 1e-9.
         max_iter (int, optional): maximum number of iterations before switching. Defaults to 100.
-    Return:
+        use_c (bool, optional): Use the C solver if configured. Defaults to False
+        use_gpu (bool, optional): Use the GPU solver if configured. Defaults to False
+
+Return:
         eanom (float/np.array): eccentric anomalies, same shape as manom
 
     Written: Jason Wang, 2018
@@ -159,41 +186,68 @@ def _calc_ecc_anom(manom, ecc, tolerance=1e-9, max_iter=100, use_c=False):
 
     # Now low eccentricities
     ind_low = np.where(~ecc_zero & ecc_low)
-    if cext and use_c:
-        if len(ind_low[0]) > 0: eanom[ind_low] = _kepler._c_newton_solver(manom[ind_low], ecc[ind_low], tolerance=tolerance, max_iter=max_iter)
-
-        # the C solver returns eanom = -1 if it doesnt converge after max_iter iterations
-        m_one = eanom == -1
-        ind_high = np.where(~ecc_zero & ~ecc_low | m_one)
-    else:
-        if len(ind_low[0]) > 0:
-            eanom[ind_low] = _newton_solver(
-                manom[ind_low], ecc[ind_low], tolerance=tolerance, max_iter=max_iter)
-        ind_high = np.where(~ecc_zero & ~ecc_low)
-
+    if len(ind_low[0]) > 0: 
+        eanom[ind_low] = _newton_solver_wrapper(manom[ind_low], ecc[ind_low], tolerance, max_iter, use_c, use_gpu)
+    
     # Now high eccentricities
+    ind_high = np.where(~ecc_zero & ~ecc_low | (eanom == -1)) # The C and CUDA solvers return the unphysical value -1 if they fail to converge
     if len(ind_high[0]) > 0: 
-        eanom[ind_high] = _mikkola_solver_wrapper(manom[ind_high], ecc[ind_high], use_c)
+        eanom[ind_high] = _mikkola_solver_wrapper(manom[ind_high], ecc[ind_high], use_c, use_gpu)
 
     return np.squeeze(eanom)[()]
 
+def _newton_solver_wrapper(manom, ecc, tolerance, max_iter, use_c=False, use_gpu=False):
+    """
+    Wrapper for the various (Python, C, CUDA) implementations of the Newton-Raphson solver 
+    for eccentric anomaly.
+
+    Args:
+        manom (np.array): array of mean anomalies
+        ecc (np.array): array of eccentricities
+        eanom0 (np.array, optional): array of first guess for eccentric anomaly, same shape as manom (optional)
+        use_c (bool, optional): Use the C solver if configured. Defaults to False
+        use_gpu (bool, optional): Use the GPU solver if configured. Defaults to False
+    Return:
+        eanom (np.array): array of eccentric anomalies
+
+    Written: Devin Cody, 2021
+    """
+    eanom = np.empty_like(manom)
+    
+    if cuda_ext and use_gpu:
+        # the CUDA solver returns eanom = -1 if it doesnt converge after max_iter iterations
+        eanom = _CUDA_newton_solver(manom, ecc, tolerance=tolerance, max_iter=max_iter)
+    elif cext and use_c:
+        # the C solver returns eanom = -1 if it doesnt converge after max_iter iterations
+        eanom = _kepler._c_newton_solver(manom, ecc, tolerance=tolerance, max_iter=max_iter)
+    else:
+        eanom = _newton_solver(manom, ecc, tolerance=tolerance, max_iter=max_iter)
+
+    return eanom
 
 def _newton_solver(manom, ecc, tolerance=1e-9, max_iter=100, eanom0=None):
     """
     Newton-Raphson solver for eccentric anomaly.
+
     Args:
         manom (np.array): array of mean anomalies
         ecc (np.array): array of eccentricities
-        eanom0 (np.array): array of first guess for eccentric anomaly, same shape as manom (optional)
+        tolerance (float, optional): absolute tolerance of iterative computation. 
+            Defaults to 1e-9.
+        max_iter (int, optional): maximum number of iterations before switching. 
+            Defaults to 100.
+        eanom0 (np.array): array of first guess for eccentric anomaly, same 
+            shape as manom (optional)
+
+
     Return:
         eanom (np.array): array of eccentric anomalies
 
     Written: Rob De Rosa, 2018
-
     """
     # Ensure manom and ecc are np.array (might get passed as astropy.Table Columns instead)
-    manom = np.array(manom)
-    ecc = np.array(ecc)
+    manom = np.asarray(manom)
+    ecc = np.asarray(ecc)
 
     # Initialize at E=M, E=pi is better at very high eccentricities
     if eanom0 is None:
@@ -221,18 +275,47 @@ def _newton_solver(manom, ecc, tolerance=1e-9, max_iter=100, eanom0=None):
 
     if niter >= max_iter:
         print(manom[ind], eanom[ind], diff[ind], ecc[ind], '> {} iter.'.format(max_iter))
-        eanom[ind] = _mikkola_solver_wrapper(manom[ind], ecc[ind], use_c) # Send remaining orbits to the analytical version, this has not happened yet...
+        eanom[ind] = _mikkola_solver_wrapper(manom[ind], ecc[ind]) # Send remaining orbits to the analytical version, this has not happened yet...
 
     return eanom
 
-def _mikkola_solver_wrapper(manom, ecc, use_c):
+def _CUDA_newton_solver(manom, ecc, tolerance=1e-9, max_iter=100, eanom0=None):
     """
-    Analtyical Mikkola solver (S. Mikkola. 1987. Celestial Mechanics, 40, 329-334.) for the eccentric anomaly.
-    Wrapper for the python implemenation of the IDL version. From Rob De Rosa.
+    Helper function for calling the CUDA implementation of the Newton-Raphson solver for eccentric anomaly.
+
+    Args:
+        manom (np.array): array of mean anomalies
+        ecc (np.array): array of eccentricities
+        eanom0 (np.array, optional): array of first guess for eccentric anomaly, same shape as manom (optional)
+    Return:
+        eanom (np.array): array of eccentric anomalies
+
+    Written: Devin Cody, 2021
+    """
+    global kep_gpu_ctx
+
+    # Ensure manom and ecc are np.array (might get passed as astropy.Table Columns instead)
+    manom = np.asarray(manom)
+    ecc = np.asarray(ecc)
+    eanom = np.empty_like(manom)
+    tolerance = np.asarray(tolerance, dtype = np.float64)
+    max_iter = np.asarray(max_iter)
+    
+    kep_gpu_ctx.newton(manom, ecc, eanom, eanom0, tolerance, max_iter)
+
+    return eanom
+
+def _mikkola_solver_wrapper(manom, ecc, use_c=False, use_gpu=False):
+    """
+    Wrapper for the various (Python, C, CUDA) implementations of Analtyical Mikkola solver 
 
     Args:
         manom (np.array): array of mean anomalies between 0 and 2pi
         ecc (np.array): eccentricity
+        use_c (bool, optional): Use the C solver if configured. Defaults to False
+        use_gpu (bool, optional): Use the GPU solver if configured. Defaults to False
+
+
     Return:
         eanom (np.array): array of eccentric anomalies
 
@@ -241,7 +324,9 @@ def _mikkola_solver_wrapper(manom, ecc, use_c):
 
     ind_change = np.where(manom > np.pi)
     manom[ind_change] = (2.0 * np.pi) - manom[ind_change]
-    if cext and use_c:
+    if cuda_ext and use_gpu:
+        eanom = _CUDA_mikkola_solver(manom, ecc)
+    elif cext and use_c:
         eanom = _kepler._c_mikkola_solver(manom, ecc)
     else:
         eanom = _mikkola_solver(manom, ecc)
@@ -252,7 +337,7 @@ def _mikkola_solver_wrapper(manom, ecc, use_c):
 
 def _mikkola_solver(manom, ecc):
     """
-    Analtyical Mikkola solver for the eccentric anomaly.
+    Analtyical Mikkola solver for the eccentric anomaly. See: S. Mikkola. 1987. Celestial Mechanics, 40, 329-334.
     Adapted from IDL routine keplereq.pro by Rob De Rosa http://www.lpl.arizona.edu/~bjackson/idl_code/keplereq.pro
 
     Args:
@@ -288,3 +373,26 @@ def _mikkola_solver(manom, ecc):
     u4 = -f/(f1+0.5*f2*u3+(1.0/6.0)*f3*u3*u3+(1.0/24.0)*f4*(u3**3.0))
 
     return (e0 + u4)
+
+def _CUDA_mikkola_solver(manom, ecc):
+    """
+    Helper function for calling the CUDA implementation of the Analtyical Mikkola solver for the eccentric anomaly.
+
+    Args:
+        manom (float or np.array): mean anomaly, must be between 0 and pi.
+        ecc (float or np.array): eccentricity
+    Return:
+        eanom (np.array): array of eccentric anomalies
+
+    Written: Devin Cody, 2021
+    """
+    global kep_gpu_ctx
+
+    # Ensure manom and ecc are np.array (might get passed as astropy.Table Columns instead)
+    manom = np.asarray(manom)
+    ecc = np.asarray(ecc)
+    eanom = np.empty_like(manom)
+
+    kep_gpu_ctx.mikkola(manom, ecc, eanom)
+
+    return eanom
